#!/usr/bin/python

from PIL import Image

def invert(value):
    return ''.join(["1" if x == "0" else "0" for x in value])


# Load Image
im = Image.open('QR3C_cropped.png')
# im.crop((33, 34, 264, 265)).save("QR3C_cropped.png")


pix = im.load()

tiles = 21
tileSize = im.size[0] // tiles
version = (tiles - 17) // 4

data = [[0 for x in range(tiles)] for y in range(tiles)]
print("tiles=%d tileSize=%d version=%d" % (tiles, tileSize, version))
for x in range(tiles):
    for y in range(tiles):
        x_mid = x * tileSize + 0.5 * tileSize
        y_mid = y * tileSize + 0.5 * tileSize
        color = pix[x_mid, y_mid]
    #    print(x, y, x_mid, y_mid, color)
        data[x][y] = invert("%d%d%d" %  ( color[0] // 255, color[1] // 255, color[2] // 255 ))

# print(data)

def blackWhiteValue(string):
    new_value = ""
    if len(string) % 3 != 0:
        print("Cannot return black white value of", string)
        return new_value

    for i in range(len(string) // 3):
        new_value += string[i * 3]

    return new_value

mask1 = [data[8][18], data[8][17], data[8][16]]
mask2 = [data[2][8], data[3][8], data[4][8]]
if mask1 != mask2:
    print("masks do not match")
    exit()

# mask = [mask1[0][0] + mask1[1][0] + mask[2][0],]
mask = [''.join(mask1[i][j] for i in range(3)) for j in range(3)]
print("Mask:", mask)

def apply_mask(x, y, bit, mask):
    doInvert = False
    if mask == "111":
        doInvert = x % 3 == 0
    elif mask == "110":
        doInvert = (x + y) % 3 == 0
    elif mask == "101":
        doInvert = (x + y) % 2 == 0
    elif mask == "100":
        doInvert = y % 2 == 0
    elif mask == "000":
        doInvert = ((x*y) % 2) + ((x + y) % 3) == 0

    return invert(bit) if doInvert else bit

def get_masked_value(x, y):
    return ''.join([apply_mask(x, y, data[x][y][i], mask[i]) for i in range(3)])

def read_block(x, y, orientation):
    indizes = []
    if orientation == "UP":
        indizes = [[0,0], [-1,0], [0,-1], [-1,-1], [0,-2], [-1,-2], [0, -3], [-1, -3]]
    elif orientation == "LEFT":
        indizes = [[0,0], [-1,0], [0,-1], [-1,-1], [-2,-1], [-3,-1], [-2, 0], [-3, 0]]

    return ''.join([get_masked_value(x + offset[0], y + offset[1]) for offset in reversed(indizes)])


encoding = blackWhiteValue(data[20][20] + data[19][20] + data[20][19] + data[19][19])
print("Encoding:", encoding)
# Alphanumeric? 11 bits per 2 character

# for y in range(tiles):
#     print(''.join(data[y]))

block1 = read_block(20, 14, "UP")
block2 = read_block(20, 10, "LEFT")
print(block1, len(block1))
print(block2, len(block2))
